"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocsCrawler = void 0;
const utils_1 = require("../utils");
const mongodb_schema_1 = require("mongodb-schema");
const jsdom_1 = require("jsdom");
const shell_bson_parser_1 = __importStar(require("@mongodb-js/shell-bson-parser"));
class DocsCrawler {
    constructor(url) {
        this.url = url;
        this.virtualConsole = new jsdom_1.VirtualConsole();
        this.virtualConsole.on('jsdomError', () => {
        });
    }
    fuzzyParse(json) {
        json = json.replace(/\.\.\.$/g, '');
        if (!json.startsWith('[')) {
            json = `[${json}`;
        }
        if (!json.endsWith(']')) {
            json = `${json}]`;
        }
        json = json.replace(/\}\s*\{/g, '},{');
        json = json.replace(/NumberDecimal\((?<number>[\d.]*)\)/g, 'NumberDecimal("$<number>")');
        try {
            let result = (0, shell_bson_parser_1.default)(json, { mode: shell_bson_parser_1.ParseMode.Loose });
            if (!Array.isArray(result)) {
                result = [result];
            }
            if (result.length > 0) {
                const firstDoc = result[0];
                if (typeof firstDoc !== 'object' ||
                    Object.keys(firstDoc).find((k) => k.startsWith('$'))) {
                    throw new Error('Result is not an array of documents or contains aggregation pipeline stages');
                }
                return result;
            }
            throw new Error(`Unexpected json output: ${result}`);
        }
        catch (e) {
            const message = e instanceof Error ? e.message : String(e);
            console.error('Failed to parse JSON', json, message);
            return undefined;
        }
    }
    getInsertionCode(element, extendedSearch = false) {
        if (!element) {
            return undefined;
        }
        const snippets = [...element.querySelectorAll('script')].slice(0, extendedSearch ? 2 : 1);
        for (const snippet of snippets) {
            let codeSnippet = JSON.parse(snippet.innerHTML).text;
            codeSnippet = (0, utils_1.removeTrailingComments)(codeSnippet);
            codeSnippet = (0, utils_1.removeNewlines)(codeSnippet);
            let collectionName = undefined;
            const insertionCode = /db(?:\..*)?\.(?<collectionName>[^.]*)\.(insertMany|insertOne)\s*\(\s*(?<documents>(?:\[|{).*(?:]|}))\s*\)/gm.exec(codeSnippet);
            if (insertionCode && insertionCode.groups) {
                collectionName = insertionCode.groups.collectionName;
                codeSnippet = insertionCode.groups.documents;
            }
            else {
                const aggregationDocs = /db\.aggregate\(\s*\[\s*{\s*\$documents:\s*(?<documents>\[.*])\s*},\s*{\s*\$/gm.exec(codeSnippet);
                if (aggregationDocs && aggregationDocs.groups) {
                    codeSnippet = aggregationDocs.groups.documents;
                }
            }
            const documents = this.fuzzyParse(codeSnippet);
            if (documents) {
                collectionName ??=
                    Array.from(element?.querySelectorAll('p') || [])
                        .find((p) => p.textContent?.trim().startsWith('Consider a'))
                        ?.querySelector('code')?.innerHTML ?? 'TestCollection';
                return {
                    collectionName,
                    documents,
                };
            }
        }
        while (element) {
            element = element?.parentElement?.closest('section');
            const examples = element?.querySelector("a[href='#examples']") ??
                element?.querySelector("a[href='#example']");
            if (examples) {
                return this.getInsertionCode(element, true);
            }
        }
        return undefined;
    }
    async getSchema() {
        const fragment = new URL(this.url).hash;
        if (!fragment) {
            return;
        }
        try {
            const dom = await jsdom_1.JSDOM.fromURL(this.url, {
                virtualConsole: this.virtualConsole,
            });
            const exampleSection = dom.window.document
                .querySelector(`a[href='${fragment}']:not([target])`)
                ?.closest('section');
            const insertionCode = this.getInsertionCode(exampleSection);
            if (!insertionCode) {
                return;
            }
            return {
                schema: await (0, mongodb_schema_1.getSimplifiedSchema)(insertionCode.documents),
                collectionName: insertionCode.collectionName,
            };
        }
        catch (e) {
            const message = e instanceof Error ? e.message : String(e);
            console.error('Failed to parse schema', this.url, message);
            return;
        }
    }
}
exports.DocsCrawler = DocsCrawler;
//# sourceMappingURL=docs-crawler.js.map