"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.DriverSchemaGenerator = void 0;
const generator_1 = require("../generator");
const metaschema_1 = require("../metaschema");
const yaml = __importStar(require("js-yaml"));
const fs = __importStar(require("fs/promises"));
const docs_crawler_1 = require("./docs-crawler");
const static_schemas_1 = require("./static-schemas");
class DriverSchemaGenerator extends generator_1.GeneratorBase {
    async getSchemaFromDocs(test) {
        if (!test.link) {
            console.error(`No docs reference found for ${test.name}`);
            return '// TODO: No docs reference found';
        }
        if (!test.pipeline) {
            console.error(`No pipeline found for ${test.name} at ${test.link}`);
            return '// TODO: No pipeline found';
        }
        const docsCrawler = new docs_crawler_1.DocsCrawler(test.link);
        const schema = await docsCrawler.getSchema();
        if (!schema) {
            console.error(`Could not extract schema for ${test.name} at ${test.link}`);
            return '// TODO: No schema found in docs';
        }
        return {
            [schema.collectionName]: schema.schema,
        };
    }
    async updateTestSchema({ category, operator, test, rawYaml, }) {
        const yamlTest = rawYaml.tests.find((t) => t.name === test.name);
        if (!yamlTest) {
            console.error(`Test ${test.name} not found in operator ${operator} in category ${category}`);
            return;
        }
        yamlTest.schema =
            (0, static_schemas_1.getStaticSchema)({ category, operator, test: test.name }) ??
                (await this.getSchemaFromDocs(test));
    }
    async generateImpl(yamlFiles) {
        for await (const file of yamlFiles) {
            for await (const operator of file.operators()) {
                const parsed = metaschema_1.Operator.parse(operator.yaml);
                const operatorYaml = operator.yaml;
                for (const test of parsed.tests ?? []) {
                    await this.updateTestSchema({
                        category: file.category,
                        operator: operatorYaml.name,
                        test,
                        rawYaml: operatorYaml,
                    });
                }
                let updatedYaml = yaml.dump(operatorYaml, {
                    schema: generator_1.GeneratorBase.loadOptions.schema,
                    indent: 4,
                    lineWidth: -1,
                });
                updatedYaml = `# $schema: ../schema.json\n${updatedYaml}`;
                await fs.writeFile(operator.path, updatedYaml, 'utf8');
            }
        }
    }
}
exports.DriverSchemaGenerator = DriverSchemaGenerator;
//# sourceMappingURL=driver-schema-generator.js.map