"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Operator = void 0;
const zod_1 = require("zod");
exports.Operator = zod_1.z
    .object({
    name: zod_1.z.string().regex(new RegExp('^\\$?[a-z][a-zA-Z0-9]*$')),
    link: zod_1.z.string().url().regex(new RegExp('^https://')),
    type: zod_1.z.array(zod_1.z.enum([
        'accumulator',
        'stage',
        'query',
        'fieldQuery',
        'filter',
        'window',
        'geometry',
        'switchBranch',
        'resolvesToAny',
        'resolvesToNumber',
        'resolvesToDouble',
        'resolvesToString',
        'resolvesToObject',
        'resolvesToArray',
        'resolvesToBinData',
        'resolvesToObjectId',
        'resolvesToBool',
        'resolvesToDate',
        'resolvesToNull',
        'resolvesToRegex',
        'resolvesToJavascript',
        'resolvesToInt',
        'resolvesToTimestamp',
        'resolvesToLong',
        'resolvesToDecimal',
        'searchOperator',
    ])),
    encode: zod_1.z.enum(['array', 'object', 'single']),
    description: zod_1.z.string(),
    wrapObject: zod_1.z.boolean().default(true),
    arguments: zod_1.z
        .array(zod_1.z
        .object({
        name: zod_1.z.string().regex(new RegExp('^([_$]?[a-z][a-zA-Z0-9]*|N)$')),
        type: zod_1.z.array(zod_1.z.enum([
            'accumulator',
            'query',
            'fieldQuery',
            'pipeline',
            'untypedPipeline',
            'window',
            'expression',
            'expressionMap',
            'geometry',
            'fieldPath',
            'unprefixedFieldPath',
            'timeUnit',
            'sortSpec',
            'any',
            'granularity',
            'fullDocument',
            'fullDocumentBeforeChange',
            'accumulatorPercentile',
            'whenMatched',
            'whenNotMatched',
            'outCollection',
            'range',
            'sortBy',
            'geoPoint',
            'resolvesToNumber',
            'numberFieldPath',
            'number',
            'resolvesToDouble',
            'doubleFieldPath',
            'double',
            'resolvesToString',
            'stringFieldPath',
            'string',
            'resolvesToObject',
            'objectFieldPath',
            'object',
            'resolvesToArray',
            'arrayFieldPath',
            'array',
            'resolvesToBinData',
            'binDataFieldPath',
            'binData',
            'resolvesToObjectId',
            'objectIdFieldPath',
            'objectId',
            'resolvesToBool',
            'boolFieldPath',
            'bool',
            'resolvesToDate',
            'dateFieldPath',
            'date',
            'resolvesToNull',
            'nullFieldPath',
            'null',
            'resolvesToRegex',
            'regexFieldPath',
            'regex',
            'resolvesToJavascript',
            'javascriptFieldPath',
            'javascript',
            'resolvesToInt',
            'intFieldPath',
            'int',
            'resolvesToTimestamp',
            'timestampFieldPath',
            'timestamp',
            'resolvesToLong',
            'longFieldPath',
            'long',
            'resolvesToDecimal',
            'decimalFieldPath',
            'decimal',
            'searchPath',
            'searchScore',
            'searchOperator',
            'searchHighlight',
        ])),
        description: zod_1.z.string().optional(),
        optional: zod_1.z.boolean().optional(),
        valueMin: zod_1.z.number().optional(),
        valueMax: zod_1.z.number().optional(),
        variadic: zod_1.z.enum(['array', 'object']).optional(),
        variadicMin: zod_1.z.number().int().gte(0).optional(),
        default: zod_1.z
            .union([zod_1.z.array(zod_1.z.any()), zod_1.z.boolean(), zod_1.z.number(), zod_1.z.string()])
            .optional(),
        mergeObject: zod_1.z.boolean().default(false),
        syntheticVariables: zod_1.z
            .array(zod_1.z.object({
            name: zod_1.z.string().regex(new RegExp('^[$]?[a-zA-Z0-9]*$')),
            description: zod_1.z.string().optional(),
        }))
            .optional(),
    })
        .strict())
        .optional(),
    tests: zod_1.z
        .array(zod_1.z
        .object({
        name: zod_1.z.string(),
        link: zod_1.z.string().url().regex(new RegExp('^https://')).optional(),
        pipeline: zod_1.z.array(zod_1.z.record(zod_1.z.any())),
        schema: zod_1.z
            .union([
            zod_1.z.string(),
            zod_1.z.record(zod_1.z.record(zod_1.z
                .object({
                types: zod_1.z.array(zod_1.z.union([
                    zod_1.z
                        .object({
                        bsonType: zod_1.z.enum([
                            'Array',
                            'Binary',
                            'Boolean',
                            'Code',
                            'CodeWScope',
                            'Date',
                            'Decimal128',
                            'Double',
                            'Int32',
                            'Int64',
                            'Long',
                            'MaxKey',
                            'MinKey',
                            'Null',
                            'ObjectId',
                            'BSONRegExp',
                            'String',
                            'BSONSymbol',
                            'Timestamp',
                            'Undefined',
                            'Document',
                            'Number',
                        ]),
                    })
                        .strict(),
                    zod_1.z
                        .object({
                        bsonType: zod_1.z.literal('Array'),
                        types: zod_1.z.array(zod_1.z.record(zod_1.z.any())),
                    })
                        .strict(),
                    zod_1.z
                        .object({
                        bsonType: zod_1.z.literal('Document'),
                        fields: zod_1.z.record(zod_1.z.record(zod_1.z.any())),
                    })
                        .strict(),
                ])),
            })
                .strict())),
        ])
            .optional(),
    })
        .strict())
        .optional(),
})
    .strict();
//# sourceMappingURL=metaschema.js.map